<?php
/**
 * @file
 * Html datalist form element file.
 */

namespace Drupal\datalist\Element;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element\FormElement;
use Drupal\Core\Template\Attribute;

/**
 * Provides a render element for a datalist.
 *
 * Usage example:
 * @code
 * $form['author']['name'] = array(
 *   '#type' => 'datalist',
 *   '#options' => [],
 *   '#title' => '',
 *   '#required' => '',
 *   '#....
 * );
 * @endcode
 *
 * @FormElement("datalist")
 */
class HtmlDatalist extends FormElement {

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    $class = static::class;

    return [
      '#input' => TRUE,
      '#theme' => 'datalist',
      '#theme_wrappers' => ['form_element'],
      '#options' => [],
      '#data_list_id' => '',
      '#pre_render' => [
        [$class, 'preRenderDatalist'],
      ],
    ];
  }

  /**
   * Pre-render callback: Renders a datalist.
   *
   * @param array $element
   *
   * @return array
   */
  public static function preRenderDatalist($element) {
    $attributes = isset($element['#attributes']) ? new Attribute($element['#attributes']) : new Attribute();

    $listId = $element['#name'] . '-options';

    $dataListAttributes = new Attribute();
    $element['#data_list_attributes']
      = $dataListAttributes->setAttribute('id', $listId);

    $attributes->setAttribute('name', 'edit-' . $element['#name']);
    $attributes->setAttribute('list', $listId);

    if ($element['#required']) {
      $attributes->setAttribute('required', 'required');
    }

    $element['#accessible_clear_text'] = t('Clear @name field', [
      '@name' => $element['#title'],
    ]);

    $element['#attributes'] = $attributes;

    $element['#attached']['library'][] = 'datalist/datalist';

    return $element;
  }

  /**
   * {@inheritdoc}
   *
   * Return the real value of the "datalist" list.
   */
  public static function valueCallback(&$element, $input, FormStateInterface $form_state) {
    $userInput = $form_state->getUserInput();
    if (isset($userInput[$element['#id']])) {
      $label = $userInput[$element['#id']];
      return array_search($label, $element['#options'], TRUE);
    }
  }

}
